#!/bin/sh
#
# if your're using this script, you have to provide an on_error function
# and you must have included functions.sh

function postgresql()
{
    if [ ! $1 ] || [ ! $2 ] || [ ! $3 ] || [ ! $4 ]; then
        log_error "Not all parameters for postgresl() function set!"
        on_error
        exit 1
    fi

    appl=$1
    cfg_db_tmp=$2
    database=$3
    password=$4

    if [ -n "$(dpkg -l | grep '^ii[ ]*postgresql-9.1')" ] ; then
        log_msg "PostgreSQL already installed"
        if [ $(pg_lsclusters | wc -l) -eq "1" ]; then
            log_error "No PostgreSQL cluster found"
            on_error
            exit 1
        else
            version=`pg_lsclusters | egrep -oe "^9\.[0-9]+"`
            if [ -z "$version" ]; then
                 log_msg "PostgreSQL Version could not be found. Using 9.1"
                 version="9.1"
            fi
            cluster=`pg_lsclusters | tr "\011" " " | egrep -oe "^9\.[0-9][ ]*[^ ]+" | sed -e "s/^[0-9\. ]\+//"`
            if [ -z "$cluster" ]; then
                 log_msg "PostgreSQL cluster name could not be found. Use main"
                 cluster="main"
            fi
            config_pg_file="postgresql.conf"
            config_pg_dir=/etc/postgresql/$version/$cluster/
            data_pg_dir=/var/lib/postgresql/$version/$cluster/
            port_number=5432	# config
        fi
    else
        log_error "PostgreSQL not installed!"
        on_error
        exit 1
    fi

    log_msg "PostgreSQL config dir: $config_pg_dir"
    log_msg "PostgreSQL configuration: $config_pg_file"
    log_msg "PostgreSQL data dir: $data_pg_dir"
    log_msg "PostgreSQL port: $port_number"
    log_msg "PostgreSQL cluster: $cluster"

    #
    # Get here config file location from postgresql_install	
    #
    pgsql_cfg="${config_pg_dir}/${config_pg_file}"
    cp $pgsql_cfg $cfg_db_tmp
    cp "${config_pg_dir}/pg_hba.conf" $cfg_db_tmp

    postgresql_tmp=`mktemp $cfg_db_tmp/pg_conf.$$.XXXX`
    if [ $? -ne 0 ]; then
        log_error "Can't create temp config file, exiting..."
        on_error
        exit 1
    fi

    if [ -e "${config_pg_dir}/pg_hba.conf" ]; then
        cat $config_pg_dir/pg_hba.conf | sed -e '/^[ ]*host.*postgres.*postgres.*127.0.0.1\/32.*md5.*/ d' -e '/^[ ]*host.*${appl}.*/ d' > $postgresql_tmp
        # Do this to check actual password through web interface
        echo "host    postgres     postgres         127.0.0.1/32             md5" >> $postgresql_tmp
        echo "host    ${appl}     postgres         127.0.0.1/32             trust" >> $postgresql_tmp
        echo "host    ${appl}     ${appl}         127.0.0.1/32             trust" >> $postgresql_tmp
        cp $postgresql_tmp $config_pg_dir/pg_hba.conf
    else
        log_error "pg_hba.conf file does not exist in $config_pg_dir"
        on_error
        exit 1
    fi

    # User in postgres group can also modify this file
    chmod 660 /etc/postgresql/9.1/main/pg_hba.conf
    chmod 775 ${database}
    if [ -z $(sudo -u postgres psql -c 'select usename from pg_user;' | grep "${appl}") ]; then
        createUser=`sudo -u postgres psql -c "CREATE USER \"${appl}\" WITH PASSWORD '${password}'" 2>/dev/null`
        if ! echo $createUser | grep "CREATE ROLE"
        then
            log_error "${appl} user could not be created in PostgreSQL"
            on_error
            exit 1
        fi
        log_msg "database user '${appl}' created"
    else
        log_error "database user '${appl}' already exists!"
        on_error
        exit 1
    fi
    database_user="${appl}"

    mv ${database} $cfg_db_tmp/db.psql
    if [ -z $(su -c 'psql -l' postgres | grep "${appl}") ]; then
        sudo -u postgres createdb --encoding=utf8 -T template0 -U postgres ${appl}
        if [ -z "$(su -c 'psql -l' postgres | grep ${appl})" ]; then
            log_error "${appl} database could not be created in PostgreSQL"
            on_error
	        exit 1
        fi
        database_name="${appl}"
    else
        log_error "database '${appl}' already exists!"
        on_error
        exit 1
    fi
    grantUser=`sudo -u postgres psql -c "GRANT ALL PRIVILEGES ON DATABASE \"${appl}\" to \"${appl}\""`
    # sudo -u postgres psql -c "REVOKE ALL PRIVILEGES ON DATABASE postgres FROM \"${appl}\""
    if ! echo $grantUser | grep "GRANT"
    then
        log_error "rights could not be granted to ${appl}"
        on_error
        exit 1
    fi
    sudo -u postgres psql -c "ALTER DATABASE \"${appl}\" OWNER TO \"${appl}\""
    sudo -u postgres psql -f $cfg_db_tmp/db.psql -d ${appl}
    /etc/init.d/postgresql restart
    if [ -z "$(ps axwww | grep postgresql | grep -v 'grep')" ]; then
        log_msg "Warning: PostgreSQL service not running"
    fi
    if [ ! -f "/home/postgres/restore" ]; then
        mkdir -p /home/postgres/restore
    fi
    chmod 755 *.psql
    cp *.psql /home/postgres/restore || log_msg "Warning: Sequences will be not updated during backups"

    if [ -d "/var/log/postgresql" ]; then
        chmod -R +r /var/log/postgresql
    fi
}